<?php
/*
FORMBUILDER SETUP
*/
// include FormBuilder class
include('formbuilder/FormBuilder.php');
// create a new instance of FormBuilder class
$contactForm = new FormBuilder('contactform','Send Email','Please complete the following form.');
// define our new FormBuilder
$contactForm->addFieldSet('Personal details');
$contactForm->addField(array(
		'id' => 'name',
		'type' => 'textbox',
		'label' => 'Name',
		'required' => true,
		'instructions' => 'These are some instructions',
		'maxlength' => 100,
		'headerinjectioncheck' => 'full'
	)
);
$contactForm->addField(array(
		'id' => 'email',
		'type' => 'textbox',
		'label' => 'Email',
		'required' => true,
		'maxlength' => 100,
		'validationtype' => 'email',
		'headerinjectioncheck' => 'full'
		//'defaultvalue' => 'Your email'
	)
);
$contactForm->addField(array(
		'id' => 'userpassword',
		'type' => 'password',
		'label' => 'Password',
		'required' => true,
		'maxlength' => 12
	)
);
$contactForm->addFieldSet('Your message');
$contactForm->addField(array(
		'id' => 'subject',
		'type' => 'select',
		'label' => 'Subject',
		'required' => true,
		'optionlabels' => array('Please select a subject','General enquiry','Booking enquiry'),
		'optionvalues' => array('Please select a subject','General enquiry','Booking enquiry'),
		'invalidvalue' => 'Please select a subject'
		//'defaultvalue' => 'General enquiry'
	)
);
$contactForm->addField(array(
		'id' => 'message',
		'type' => 'textarea',
		'label' => 'Message',
		'required' => true,
		'rows' => 5,
		'cols' => 35,
		'minlength' => 3,
		'maxlength' => 200,
		'headerinjectioncheck' => 'light'
		//'defaultvalue' => 'Your messsage'
	)
);
$contactForm->addFieldSet('Additional information');
$contactForm->addField(array(
		'id' => 'mailinglist',
		'type' => 'radio',
		'label' => 'Add me to your mailing list',
		'required' => true,
		'radiolabels' => array('Yes','No'),
		'radiovalues' => array('y','n'),
		'defaultvalue' => 'n'
	)
);
$contactForm->addField(array(
		'id' => 'interests',
		'type' => 'checkbox',
		'label' => 'Interests',
		'required' => true,
		'checkboxlabels' => array('Music','Computers','Movies','Sport','Art'),
		'checkboxvalues' => array('Music','Computers','Movies','Sport','Art'),
		'checkboxchecked' => array(true,true,true,true,true)
	)
);
// Example of a file upload.
// Files can be accessed using PHP's $_FILES array.
//$contactForm->addField(array(
//		'id' => 'fileupload',
//		'type' => 'file',
//		'label' => 'File upload',
//		'required' => false
//	)
//);
$contactForm->addField(array(
		'id' => 'useragent',
		'type' => 'hidden',
		'value' => $_SERVER['HTTP_USER_AGENT']
	)
);
if($contactForm->isSubmitted()){
	// You may also want to do additional form validation of your own, after FormBuilder has finished.
	// If so, you can force error messages to be displayed to the user as such:
	//$contactForm->forceErrorMessage('name',"Hey! I'm sure you did something BAD with the Name field!");
}
if($contactForm->formSuccess()){
	// Put your form submission logic here. ie: send an email, make a database insert etc.
	// Access variables in the usual manner, ie: $_POST[field_id]
	//
	// Below is demonstrated the method to automatically send variables to an email address.
	// Please note that it is VITAL to have applied a full header injection check to all user
	// supplied values passed as parameters to this method (see FormBuilder documentation)
	$contactForm->emailResults(
		'you@yourdomain.com', // The recipient (ie: you). Should ALWAYS be hard coded.
		'sender@sender.com', // Sender's email. Can be a variable from the form, in the manner: $_POST['email']
		'Sender Name', // Sender's name. Can be a variable from the form, in the manner: $_POST['name']
		'Pixel Acres feedback', // The email subject. Can be a variable from the form, in the manner: $_POST['subject']
		'The following is an email sent by a visitor to Pixel Acres:' // The email intro message
	);
}
?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />
<title>FormBuilder example</title>
<!-- PAGE CSS STYLES -->
<link rel="StyleSheet" href="main.css" type="text/css" media="screen" />
<!-- FORM BUILDER CSS STYLES -->
<link rel="StyleSheet" href="formbuilder/formbuilder.css" type="text/css" media="screen" />
</head>

<body>

<div id="container">
	<h1>FormBuilder example - semantic</h1>
<?php
$contactForm->renderThanks(null,null,true);
$contactForm->renderErrorMessage();
?>
<?php
if(!$contactForm->formSuccess()){
	$contactForm->renderForm();
}
?>
<?php
if($contactForm->formSuccess()){
	// This is just for demonstration purpoes.
	// Submitted variables will be echoed to the screen.
	// Access variables in the usual manner, ie: $_POST[field_id]
	echo('<h2>Sumitted form variables</h2>');
	echo('<p>The submitted form had the values:</p>');
	echo('<p><strong>Name:</strong> '.$_POST['name'].'<br />');
	echo('<strong>Email:</strong> '.$_POST['email'].'<br />');
	echo('<strong>Password:</strong> '.$_POST['userpassword'].'<br />');
	echo('<strong>Subject:</strong> '.$_POST['subject'].'<br />');
	echo('<strong>Message:</strong> '.str_replace("\r","<br />",$_POST['message']).'<br />');
	echo('<strong>Join mailing list:</strong> '.$_POST['mailinglist'].'<br />');
	echo('<strong>Interests:</strong> '.$_POST['interests'].'<br />');
	echo('<strong>User agent (hidden field):</strong> '.$_POST['useragent']);
	echo('</p>');
}
?>

</div>

</body>
</html>